import { Routes, Route } from "react-router-dom";
import "./App.css";
import Main from "views/Main";
import Authentication from "views/Authentication";
import Search from "views/Search";
import UserP from "views/User";
import BoardDetail from "views/Board/Detail";
import BoardWrite from "views/Board/Write";
import BoardUptdade from "views/Board/Update";
import Container from "layouts/Container";
import NoticeList from "views/Menu/notice/list";
import {
  EMPLOYEE_MANAGEMENT_PATH,
  MAIN_PATH,
  NOTICE_LIST_PATH,
  NOTICE_WRITE_PATH,
} from "constant";
import { AUTH_PATH } from "constant";
import { SEARCH_PATH } from "constant";
import { USER_PATH } from "constant";
import { BOARD_PATH } from "constant";
import { BOARD_WRITE_PATH } from "constant";
import { BOARD_DETAIL_PATH } from "constant";
import { BOARD_UPDATE_PATH } from "constant";
import { useEffect } from "react";
import { useCookies } from "react-cookie";
import { useLoginuserStore } from "stores";
import { getSignInUserRequest } from "apis";
import { GetSignInUserResponseDto } from "apis/response/user";
import { ResponseDto } from "apis/response";
import User from "types/interface/user.interface";
import Employee from "views/Menu/employee";
import NoticeWrite from "views/Menu/notice/write";

//          component: 어플리케이션 컴포넌트          //
function App() {
  //          state: 로그인 유저 전역 상태           //
  const { setLoginUser, resetLoginUser } = useLoginuserStore();

  //          state: cookie 상태           //
  const [cookies, setCookie] = useCookies();

  //          function: get sign in user response 처리 함수            //
  const getSignInUserResponse = (
    responseBody: GetSignInUserResponseDto | ResponseDto | null
  ) => {
    if (!responseBody) return;
    const { code } = responseBody;
    if (code === "AF" || code === "NU" || code === "DBE") {
      resetLoginUser();
      return;
    }
    const loginUser: User = { ...(responseBody as GetSignInUserResponseDto) };
    setLoginUser(loginUser);
  };

  //          effect: accessToken cookie 값이 변경될 때 마다 실행할 함수           //
  useEffect(() => {
    if (!cookies.accessToken) {
      resetLoginUser();
      return;
    }
    getSignInUserRequest(cookies.accessToken).then(getSignInUserResponse);
  }, [cookies.accessToken]);

  //          render: 어플리케이션 컴포넌트 렌더링         //
  //          discription: 메인 화면 : '/' - Main          //
  //          discription: 로그인 화면 + 회원가입 화면: '/auth' - Authentication          //
  //          discription: 검색 화면 : '/search/:searchword' - Search          //
  //          discription: 유저 페이지 : '/user/:email' - User          //
  //          discription: 게시물 상세보기 : '/board/detail/:boardNumber' - BoardDetail          //
  //          discription: 게시물 작성하기 : '/board/write' - BoardWrite          //
  //          discription: 게시물 수정하기 : '/board/update/:boardNumber' - BoardUpdate          //
  //          discription: 사원 추가 : '/employee/sign-in' - Employee          //
  //          discription: 공지 목록 : '/motice/list' - Notice          //

  return (
    <Routes>
      <Route element={<Container />}>
        <Route path={MAIN_PATH()} element={<Main />} />
        <Route path={AUTH_PATH()} element={<Authentication />} />
        <Route path={SEARCH_PATH(":searchWord")} element={<Search />} />
        <Route path={USER_PATH(":userEmail")} element={<UserP />} />
        <Route path={EMPLOYEE_MANAGEMENT_PATH()} element={<Employee />} />
        <Route path={NOTICE_LIST_PATH()} element={<NoticeList />} />
        <Route path={NOTICE_WRITE_PATH()} element={<NoticeWrite />} />

        <Route path={BOARD_PATH()}>
          <Route path={BOARD_WRITE_PATH()} element={<BoardWrite />} />
          <Route
            path={BOARD_DETAIL_PATH(":boardNumber")}
            element={<BoardDetail />}
          />
          <Route
            path={BOARD_UPDATE_PATH(":boardNumber")}
            element={<BoardUptdade />}
          />
        </Route>
        <Route path="*" element={<h1>404 에러 발생...</h1>} />
      </Route>
    </Routes>
  );
}

export default App;
