import { useEffect, useState } from "react";
/* use로 시작하면 costom hook 함수 */
const usePagination = <T>(countPerPage: number) => {
  /* 매개타입 T -> 함수가 호출될 경우 호출되었을때 필요한 타입으로 지정 */
  //              state: 전체 객체 리스트 상태               //
  const [totalList, setTotalList] = useState<T[]>([]);
  //              state: 보여줄 객체 리스트 상태               //
  const [viewList, setViewList] = useState<T[]>([]);
  //              state: 현재 페이지 번호 상태               //
  const [currentPage, setCurrentPage] = useState<number>(1);

  //              state: 전체 페이지 번호 상태               //
  const [totalPageList, setTotalPageList] = useState<number[]>([1]);
  //              state: 보여줄 페이지 번호 리스트 상태               //
  const [viewPageList, setViewPageList] = useState<number[]>([1]);

  //              state: 현재 섹션 상태               //
  const [currentSection, setCurrentSection] = useState<number>(1);
  //              state: 전체 섹션 상태               //
  const [totalSection, setTotalSection] = useState<number>(1);

  //          function: 보여줄 객체의 리스트 추출 함수           //
  const setView = () => {
    const FIRST_INDEX = countPerPage * (currentPage - 1);
    const LAST_INDEX =
      totalList.length > countPerPage * currentPage
        ? countPerPage * currentPage
        : totalList.length;
    const viewList = totalList.slice(FIRST_INDEX, LAST_INDEX);
    setViewList(viewList);
  };

  //          function: 보여줄 페이지 리스트 추출 함수           //
  const setViewPage = () => {
    const FIRST_INDEX = 10 * (currentSection - 1);
    const LAST_INDEX =
      totalPageList.length > 10 * currentSection
        ? 10 * currentSection
        : totalPageList.length;
    const viewPageList = totalPageList.slice(FIRST_INDEX, LAST_INDEX);
    setViewPageList(viewPageList);
  };

  //          effect: total list가 변경될 때마다 실행될 작업          //
  useEffect(() => {
    const totalPage = Math.ceil(totalList.length / countPerPage);
    const totalPageList: number[] = [];
    for (let Page = 1; Page <= totalPage; Page++) totalPageList.push(Page);
    setTotalPageList(totalPageList);

    const totalSection = Math.ceil(
      totalList.length / (countPerPage * 10)
    ); /* 10개의 섹션을 보여줌 */
    setTotalSection(totalSection);

    setCurrentPage(1);
    setCurrentSection(1);

    setView();
    setViewPage();
  }, [totalList]);

  //          effect: current page가 변경될 때마다 실행될 작업          //
  useEffect(() => {
    setView();
  }, [currentPage]);

  //          effect: current section이 변경될 때마다 실행될 작업          //
  useEffect(() => {
    setViewPage();
  }, [currentPage]);

  return {
    currentPage /* 현재 페이지가 어떤 위치에 있는지 */,
    setCurrentPage,
    currentSection,
    setCurrentSection,
    viewList /* 현재 보여줄 리스트 */,
    viewPageList,
    totalSection /* 전체 섹션이 몇개인지 */,
    setTotalList,
  };
};

export default usePagination;
