import React, {
  useState,
  KeyboardEvent,
  useRef,
  ChangeEvent,
  useEffect,
} from "react";
import "./style.css";
import InputBox from "components/InputBox";
import { SignUpRequestDto } from "apis/request/auth";
import { employeeSignUpRequest } from "apis";

import { ResponseDto } from "apis/response";

import { Address, useDaumPostcodePopup } from "react-daum-postcode";
import SignUpResponseDto from "apis/response/auth/sign-up-response.dto";
import { MAIN_PATH } from "constant";
import { useNavigate } from "react-router-dom";

export default function Employee() {
  /*   //          state: 화면 상태               //
    const [view, setView] = useState<"sign-in" | "sign-up">("sign-up"); */

  /*   //          state: 쿠키 상태               //
    const [cookies, setCookie] = useCookies(); */

  //          function: 네비게이터 함수          //
  const navigate = useNavigate();

  //          component sign up card 컴포넌트          //
  const SignUpCard = () => {
    //          state: 이메일 요소 참조 상태           //
    const emailRef = useRef<HTMLInputElement | null>(null);

    //          state: 패스워드 요소 참조 상태           //
    const passwordRef = useRef<HTMLInputElement | null>(null);

    //          state: 패스워드 확인 요소 참조 상태           //
    const passCheckRef = useRef<HTMLInputElement | null>(null);

    //          state: 닉네임 요소 참조 상태           //
    const nicknameRef = useRef<HTMLInputElement | null>(null);

    //          state: 전화번호 요소 참조 상태           //
    const telNumberRef = useRef<HTMLInputElement | null>(null);

    //          state: 주소 요소 참조 상태           //
    const addressRef = useRef<HTMLInputElement | null>(null);

    //          state: 상세 주소 요소 참조 상태           //
    const addressDetailRef = useRef<HTMLInputElement | null>(null);

    //          state: 페이지 번호 상태           //
    const [page, setPage] = useState<1 | 2>(1);

    //          state: 이메일 상태           //
    const [email, setEmail] = useState<string>("");

    //          state: 패스워드 상태           //
    const [password, setPassword] = useState<string>("");

    //          state: 패스워드 확인 상태           //
    const [passCheck, setPassCheck] = useState<string>("");

    //          state: 닉네임 상태           //
    const [nickname, setNickname] = useState<string>("");

    //          state: 휴대폰 번호 상태           //
    const [telNumber, setTelNumber] = useState<string>("");

    //          state: 주소 상태           //
    const [address, setAddress] = useState<string>("");

    //          state: 상세주소 상태           //
    const [addressDetail, setAddressDetail] = useState<string>("");

    //          state: 개인 정보 동의 타입 상태           //
    const [agreedPersonal, setAgreedPersonal] = useState<boolean>(false);

    //          state: 패스워드 타입 상태           //
    const [passwordType, setPasswordType] = useState<"text" | "password">(
      "password"
    );

    //          state: 패스워드 확인 타입 상태           //
    const [passCheckType, setPassCheckType] = useState<"text" | "password">(
      "password"
    );
    //          state: 이메일 에러 상태           //
    const [isemailError, setEmailError] = useState<boolean>(false);

    //          state: 패스워드 에러 상태           //
    const [isPasswordError, setPasswordError] = useState<boolean>(false);

    //          state: 패스워드 확인 에러 상태           //
    const [isPassCheckError, setPassCheckError] = useState<boolean>(false);

    //          state: 이메일 에러 메시지 상태           //
    const [emailErrorMessage, setEmailErrorMessage] = useState<string>("");

    //          state: 패스워드 에러 메시지 상태           //
    const [passwordErrorMessage, setPasswordErrorMessage] =
      useState<string>("");

    //          state: 패스워드 확인 에러 메시지 상태           //
    const [passCheckErrorMessage, setPassCheckErrorMessage] =
      useState<string>("");

    //          state: 닉네임 에러 상태           //
    const [isNicknameError, setNicknameError] = useState<boolean>(false);

    //          state: 휴대폰 번호 에러 상태           //
    const [isTelNumberError, setTelNumberError] = useState<boolean>(false);

    //          state: 주소 에러 상태           //
    const [isAddressError, setAddressError] = useState<boolean>(false);

    //          state: 개인 정보 동의 에러 상태           //
    const [isAgreedPersonalError, setAgreedPersonalError] =
      useState<boolean>(false);

    //          state: 닉네임 에러 메시지 상태           //
    const [nicknameErrorMessage, setNicknameErrorMessage] =
      useState<string>("");

    //          state: 휴대폰 에러 메시지 상태           //
    const [telNumberErrorMessage, setTelNumberErrorMessage] =
      useState<string>("");

    //          state: 주소 에러 메시지 상태           //
    const [addressErrorMessage, setAddressErrorMessage] = useState<string>("");

    //          state: 패스워드 버튼 아이콘 상태              //
    const [passwordButtonIcon, setPasswordButtonIcon] = useState<
      "eye-light-off-icon" | "eye-light-on-icon"
    >("eye-light-off-icon");

    //          state: 패스워드 확인 버튼 아이콘 상태              //
    const [passCheckButtonIcon, setPassCheckButtonIcon] = useState<
      "eye-light-off-icon" | "eye-light-on-icon"
    >("eye-light-off-icon");

    //          function: 다음 주소 검색 팝업 오픈 함수         //
    const open = useDaumPostcodePopup();

    //          function: sign up response 처리 함수         //
    const signUpResponse = (
      responsBody: SignUpResponseDto | ResponseDto | null
    ) => {
      if (!responsBody) {
        alert("네트워크 이상입니다.");
        return;
      }

      const { code } = responsBody;
      if (code === "DE") {
        setEmailError(true);
        setEmailErrorMessage("중복되는 이메일 주소입니다.");
      }
      if (code === "DN") {
        setNicknameError(true);
        setNicknameErrorMessage("중복되는 닉네임입니다.");
      }
      if (code === "DT") {
        setTelNumberError(true);
        setTelNumberErrorMessage("중복되는 휴대폰 번호입니다.");
      }
      if (code === "VF") {
        alert("모든 값을 입력하세요.");
      }
      if (code === "DBE") {
        alert("데이터 베이스 오류입니다.");
      }
      if (code !== "SU") return;
    };

    //          event handler: 닉네임 변경 이벤트 처리           //
    const onNicknameChangeHandler = (event: ChangeEvent<HTMLInputElement>) => {
      const { value } = event.target;
      setNickname(value);
      setNicknameError(false);
      setNicknameErrorMessage("");
    };

    //          event handler: 휴대폰 번호 변경 이벤트 처리           //
    const onTelNumberChangeHandler = (event: ChangeEvent<HTMLInputElement>) => {
      const { value } = event.target;
      setTelNumber(value);
      setTelNumberError(false);
      setTelNumberErrorMessage("");
    };

    //          event handler: 주소 변경 변경 이벤트 처리           //
    const onAddressChangeHandler = (event: ChangeEvent<HTMLInputElement>) => {
      const { value } = event.target;
      setAddress(value);
      setAddressError(false);
      setAddressErrorMessage("");
    };

    //          event handler: 상세 주소 변경 이벤트 처리           //
    const onAddressDetailChangeHandler = (
      event: ChangeEvent<HTMLInputElement>
    ) => {
      const { value } = event.target;
      setAddressDetail(value);
    };
    //          event handler: 개인 정보 동의 체크박스 클릭 이벤트 처리           //
    const onAgreedPersonalClickHandler = () => {
      setAgreedPersonal(!agreedPersonal);
      setAgreedPersonalError(false);
    };

    //          event handler: 패스워드 버튼 클릭 이벤트 처리            //
    const onPasswordButtonClickHandler = () => {
      if (passwordButtonIcon === "eye-light-off-icon") {
        setPasswordButtonIcon("eye-light-on-icon");
        setPasswordType("text");
      } else {
        setPasswordButtonIcon("eye-light-off-icon");
        setPasswordType("password");
      }
    };

    //          event handler: 패스워드 확인 버튼 클릭 이벤트 처리            //
    const onPassCheckButtonClickHandler = () => {
      if (passCheckButtonIcon === "eye-light-off-icon") {
        setPassCheckButtonIcon("eye-light-on-icon");
        setPassCheckType("text");
      } else {
        setPassCheckButtonIcon("eye-light-off-icon");
        setPassCheckType("password");
      }
    };

    //          event handler: 이메일 변경 이벤트 처리           //
    const onEmailChangeHandler = (event: ChangeEvent<HTMLInputElement>) => {
      const { value } = event.target;
      setEmail(value);
      setEmailError(false);
      setEmailErrorMessage("");
    };

    //          event handler: 패스워드 변경 이벤트 처리           //
    const onPasswordChangeHandler = (event: ChangeEvent<HTMLInputElement>) => {
      const { value } = event.target;
      setPassword(value);
      setPasswordError(false);
      setPasswordErrorMessage("");
    };

    //          event handler: 패스워드 확인 변경 이벤트 처리           //
    const onPassCheckChangeHandler = (event: ChangeEvent<HTMLInputElement>) => {
      const { value } = event.target;
      setPassCheck(value);
      setPassCheckError(false);
      setPassCheckErrorMessage("");
    };

    //          event Handler: 다음 버튼 클릭 이벤트 처리           //
    const onNextButtonClickHandler = () => {
      const emailPattern = /^[a-zA-Z0-9]*@([-.]?[a-zA-Z0-9])*\.[a-zA-Z]{2,4}$/;
      const isEmailPattern = emailPattern.test(email);
      if (!isEmailPattern) {
        setEmailError(true);
        setEmailErrorMessage("이메일 주소 포멧이 맞지 않습니다.");
      }
      const isCheckedPassword = password.trim().length >= 8;
      if (!isCheckedPassword) {
        setPasswordError(true);
        setPasswordErrorMessage("비밀번호는 8자 이상 입력해 주세요.");
      }
      const isEqualPassword = password === passCheck;
      if (!isEqualPassword) {
        setPassCheckError(true);
        setPassCheckErrorMessage("비밀번호가 일지하지 않습니다.");
      }

      if (!isEmailPattern || !isCheckedPassword || !isEqualPassword) return;

      setPage(2);
    };

    //          event handler: 사원 추가 버튼 클릭 이벤트 처리          //
    const onSignUpButtonClickHandler = () => {
      const emailPattern = /^[a-zA-Z0-9]*@([-.]?[a-zA-Z0-9])*\.[a-zA-Z]{2,4}$/;
      const isEmailPattern = emailPattern.test(email);
      if (!isEmailPattern) {
        setEmailError(true);
        setEmailErrorMessage("이메일 주소 포멧이 맞지 않습니다.");
      }
      const isCheckedPassword = password.length >= 8;
      if (!isCheckedPassword) {
        setPasswordError(true);
        setPasswordErrorMessage("비밀번호는 8자 이상 입력해 주세요.");
      }
      const isEqualPassword = password === passCheck;
      if (!isEqualPassword) {
        setPassCheckError(true);
        setPassCheckErrorMessage("비밀번호가 일지하지 않습니다.");
      }
      if (!isEmailPattern || !isCheckedPassword || !isEqualPassword) {
        setPage(1);
        return;
      }
      const hasNickname = nickname.length > 0; /* nickname.trim().length  지움*/
      if (!hasNickname) {
        setNicknameError(true);
        setNicknameErrorMessage("닉네임을 입력해주세요.");
      }

      const telNumberPattern = /^[0-9]{11,13}$/;
      const isTelNumberPattern = telNumberPattern.test(telNumber);
      if (!isTelNumberPattern) {
        setTelNumberError(true);
        setTelNumberErrorMessage("숫자만 입력해주세요.");
      }

      const hasAddress = address.length > 0; /* address.trim().length  지움*/
      if (!hasAddress) {
        setAddressError(true);
        setAddressErrorMessage("주소를 선택해주세요");
      }
      if (!agreedPersonal) {
        setAgreedPersonalError(true);
      }

      if (!hasNickname || !isTelNumberPattern || !agreedPersonal) return;

      const requestBody: SignUpRequestDto = {
        email,
        password,
        nickname,
        telNumber,
        address,
        addressDetail,
        agreedPersonal,
      };

      employeeSignUpRequest(requestBody).then(signUpResponse);
      navigate(MAIN_PATH());
    };

    //          event handler: 로그인 링크 클릭 이벤트 처리          //
    const onSignInLinkClickHandler = () => {};

    //          event handler: 이메일 키 다운 이벤트 처리           //
    const onEmailKeyDownHandler = (event: KeyboardEvent<HTMLInputElement>) => {
      if (event.key !== "Enter") return;
      if (!passwordRef.current) return;
      passwordRef.current.focus();
    };

    //          event handler: 패스워드키 다운 이벤트 처리           //
    const onPasswordKeyDownHandler = (
      event: KeyboardEvent<HTMLInputElement>
    ) => {
      if (event.key !== "Enter") return;
      if (!passCheckRef.current) return;
      passCheckRef.current.focus();
    };

    //          event handler: 패스워드 확인 키 다운 이벤트 처리           //
    const onPassCheckKeyDownHandler = (
      event: KeyboardEvent<HTMLInputElement>
    ) => {
      if (event.key !== "Enter") return;
      onNextButtonClickHandler();
    };

    //          event handler: 닉네임 키 다운 이벤트 처리           //
    const onNicknameKeyDownHandler = (
      event: KeyboardEvent<HTMLInputElement>
    ) => {
      if (event.key !== "Enter") return;
      if (!telNumberRef.current) return;
      telNumberRef.current.focus();
    };

    //          event handler: 폰 번호 키 다운 이벤트 처리           //
    const onTelNumberKeyDownHandler = (
      event: KeyboardEvent<HTMLInputElement>
    ) => {
      if (event.key !== "Enter") return;
      onAddressButtonClickHandler();
    };

    //          event handler: 주소 키 다운 이벤트 처리           //
    const onAddressKeyDownHandler = (
      event: KeyboardEvent<HTMLInputElement>
    ) => {
      if (event.key !== "Enter") return;
      if (!addressDetailRef.current) return;
      addressDetailRef.current.focus();
    };

    //          event handler: 상세주소 키 다운 이벤트 처리           //
    const onAddressDetailKeyDownHandler = (
      event: KeyboardEvent<HTMLInputElement>
    ) => {
      if (event.key !== "Enter") return;
      onSignUpButtonClickHandler();
    };

    //          event handler: 다음 주소 검색 완료 이벤트 처리           //
    const onComplete = (data: Address) => {
      const { address } = data;
      setAddress(address);
      setAddressError(false);
      setAddressErrorMessage("");
      if (!addressDetailRef.current) return;
      addressDetailRef.current.focus();
    };

    //          event Handler: 주소 버튼 클릭 이벤트 처리           //
    const onAddressButtonClickHandler = () => {
      open({ onComplete });
    };

    //           effect: 페이지가 변경될 때 마다 실행될 함수           //
    useEffect(() => {
      if (page == 2) {
        if (!nicknameRef.current) return;
        nicknameRef.current.focus();
      }
    }, [page]);

    //          render: Sign up card 컴포넌트 렌더링          //
    return (
      <div className="employee-card">
        <div className="employee-card-box">
          <div className="employee-card-top">
            <div className="employee-card-title-box">
              <div className="employee-card-title">{"사용자 계정 생성"}</div>
              <div className="employee-catd-page">{`${page}/2`}</div>
            </div>
            {page === 1 && (
              <>
                <InputBox
                  ref={emailRef}
                  label="이메일 주소*"
                  type="text"
                  placeholder="이메일 주소를 입력해주세요."
                  value={email}
                  onChange={onEmailChangeHandler}
                  error={isemailError}
                  message={emailErrorMessage}
                  onKeyDown={onEmailKeyDownHandler}
                />
                <InputBox
                  ref={passwordRef}
                  label="비밀번호*"
                  type={passwordType}
                  placeholder="비밀번호를 입력해주세요."
                  value={password}
                  onChange={onPasswordChangeHandler}
                  error={isPasswordError}
                  message={passwordErrorMessage}
                  icon={passwordButtonIcon}
                  onButtonClick={onPasswordButtonClickHandler}
                  onKeyDown={onPasswordKeyDownHandler}
                />
                <InputBox
                  ref={passCheckRef}
                  label="비밀번호 확인*"
                  type={passCheckType}
                  placeholder="비밀번호를 다시 입력해주세요."
                  value={passCheck}
                  onChange={onPassCheckChangeHandler}
                  error={isPassCheckError}
                  message={passCheckErrorMessage}
                  icon={passCheckButtonIcon}
                  onButtonClick={onPassCheckButtonClickHandler}
                  onKeyDown={onPassCheckKeyDownHandler}
                />
              </>
            )}
            {page === 2 && (
              <>
                <InputBox
                  ref={nicknameRef}
                  label="닉네임*"
                  type="text"
                  placeholder="닉네임을 입력해주세요."
                  value={nickname}
                  onChange={onNicknameChangeHandler}
                  error={isNicknameError}
                  message={nicknameErrorMessage}
                  onKeyDown={onNicknameKeyDownHandler}
                />
                <InputBox
                  ref={telNumberRef}
                  label="휴대폰 번호*"
                  type="text"
                  placeholder="휴대폰 번호를 입력해주세요."
                  value={telNumber}
                  onChange={onTelNumberChangeHandler}
                  error={isTelNumberError}
                  message={telNumberErrorMessage}
                  onKeyDown={onTelNumberKeyDownHandler}
                />
                <InputBox
                  ref={addressRef}
                  label="주소*"
                  type="text"
                  placeholder="주소 찾기"
                  value={address}
                  onChange={onAddressChangeHandler}
                  error={isAddressError}
                  message={addressErrorMessage}
                  icon="expand-right-light-icon"
                  onButtonClick={onAddressButtonClickHandler}
                  onKeyDown={onAddressKeyDownHandler}
                />
                <InputBox
                  ref={addressDetailRef}
                  label="상세 주소*"
                  type="text"
                  placeholder="상세 주소를 입력해주세요."
                  value={addressDetail}
                  onChange={onAddressDetailChangeHandler}
                  error={false}
                  onKeyDown={onAddressDetailKeyDownHandler}
                />
              </>
            )}
          </div>
          <div className="employee-card-bottom">
            {page === 1 && (
              <div
                className="black-large-full-button"
                onClick={onNextButtonClickHandler}
              >
                {"다음단계"}
              </div>
            )}

            {page === 2 && (
              <>
                <div className="employee-consent-box">
                  <div
                    className="employee-check-box"
                    onClick={onAgreedPersonalClickHandler}
                  >
                    <div
                      className={`icon ${
                        agreedPersonal
                          ? "check-round-fill-icon"
                          : "check-ring-light-icon"
                      }`}
                    ></div>
                  </div>
                  <div
                    className={
                      isAgreedPersonalError
                        ? "employee-consent-title-error"
                        : "employee-consent-title"
                    }
                  >
                    {"개인정보동의"}
                  </div>
                  <div className="employee-consent-link">{"더보기 >"}</div>
                </div>
                <div
                  className="black-large-full-button"
                  onClick={onSignUpButtonClickHandler}
                >
                  {"사용자 생성"}
                </div>
              </>
            )}
          </div>
        </div>
      </div>
    );
  };

  //          render: 인증화면 컴포넌트 렌더링          //
  return (
    <div id="employee-wrapper">
      <div className="employee-container">
        <div className="employee-jumbotron-box">
          <div className="employee-jumbotron-contents">
            <div className="employee-logo-icon"></div>
            <div className="employee-jumbotron-text-box">
              <div className="employee-jumbotron-text">{""}</div>
              <div className="employee-jumbotron-text">{"  "}</div>
            </div>
          </div>
        </div>
        <SignUpCard />
      </div>
    </div>
  );
}
