import React, { useEffect, useState } from "react";
import "./style.css";
import Top3Item from "components/Top3Item";
import { BoardListItem } from "types/interface";
import BoardItem from "components/BoardItem";
import Pagination from "components/Pagination";
import { useNavigate, useParams } from "react-router-dom";
import { MAIN_PATH, SEARCH_PATH } from "constant";
import {
  getLatestBoardListRequset,
  getPopularListRequest,
  getTop3BoardListRequest,
  getUserRequest,
} from "apis";
import {
  GetLatestBoardListResponseDto,
  GetTop3BoardListResponseDto,
} from "apis/response/board";
import { ResponseDto } from "apis/response";
import { usePagination } from "hooks";
import { GetPoplarListResponseDto } from "apis/response/search";
import { useCookies } from "react-cookie";
import { GetUserResponseDto } from "apis/response/user";
import { useLoginuserStore } from "stores";

import DefaultProfileImage from "assets/image/default-profile-image.png";

//          component 메인 화면 컴포넌트           //
export default function NoticeList() {
  //          function: 네비게이트 함수           //
  const navigate = useNavigate();

  //          component 메인 화면 하단 컴포넌트           //
  const NoticeList = () => {
    //           state: 페이지네이션 관련 상태           //
    const {
      currentPage /* 현재 페이지가 어떤 위치에 있는지 */,
      setCurrentPage,
      currentSection,
      setCurrentSection,
      viewList /* 현재 보여줄 리스트 */,
      viewPageList,
      totalSection /* 전체 섹션이 몇개인지 */,
      setTotalList,
    } = usePagination<BoardListItem>(5); /* 5개씩 게시물 보여짐 */
    //           state: 인기 검색어 리스트 상태           //
    const [popularWordList, setPopularWordList] = useState<string[]>([]);

    //          function: getLatestBoardListResponse 처리함수            //
    const getLatestBoardListResponse = (
      responseBody: GetLatestBoardListResponseDto | ResponseDto | null
    ) => {
      if (!responseBody) return;
      const { code } = responseBody;
      if (code === "DBE") alert("데이터베이스 오류입니다.");
      if (code !== "SU") return;

      const { latestList } = responseBody as GetLatestBoardListResponseDto;
      setTotalList(latestList);
    };

    //          function: getPopularListResponse 처리함수             //
    const getPopularListResponse = (
      responseBody: GetPoplarListResponseDto | ResponseDto | null
    ) => {
      if (!responseBody) return;
      const { code } = responseBody;
      if (code === "DBE") alert("데이터베이스 오류입니다.");
      if (code !== "SU") return;

      const { popularWordList } = responseBody as GetPoplarListResponseDto;
      setPopularWordList(popularWordList);
    };

    //          event handler: 인기 검색어 클릭 이벤트 처리             //
    const onPopularWordClickHandler = (word: string) => {
      navigate(SEARCH_PATH(word));
    };

    //          effect: 첫 마운트 시 실행될 함수            //
    useEffect(() => {
      getLatestBoardListRequset().then(getLatestBoardListResponse);
      getPopularListRequest().then(getPopularListResponse);
    }, []);

    //          render: 메인 화면 하단 컴포넌트 렌더링         //
    return (
      <div id="notice-bottom-wrapper">
        <div className="notice-bottom-container">
          <div className="notice-bottom-title">{"공지 사항 리스트"}</div>
          <div className="notice-bottom-contents-box">
            <div className="notice-bottom-current-contents">
              {viewList.map((boardListItem) => (
                <BoardItem boardListItem={boardListItem} />
              ))}
            </div>
            <div className="notice-bottom-popular-box">
              <div className="notice-bottom-popular-card">
                <div className="notice-bottom-popular-card-box">
                  <div className="notice-bottom-popular-card-title">
                    {"인기 검색어"}
                  </div>
                  <div className="notice-bottom-popular-card-contents">
                    {popularWordList.map((word) => (
                      <div
                        className="word-badge"
                        onClick={() => onPopularWordClickHandler(word)}
                      >
                        {word}
                      </div>
                    ))}
                  </div>
                </div>
              </div>
            </div>
          </div>
          <div className="notice-bottom-pagination-box">
            <Pagination
              currentPage={currentPage}
              currentSection={currentSection}
              setCurrentPage={setCurrentPage}
              setCurrentSection={setCurrentSection}
              viewPageList={viewPageList}
              totalSection={totalSection}
            />
          </div>
        </div>
      </div>
    );
  };
  //          render 메인화면 컴포넌트 렌더링           //
  return (
    <>
      <NoticeList />
    </>
  );
}
